package ru.kristall.report;

import java.io.IOException;
import java.io.InputStream;
import java.net.JarURLConnection;
import java.net.URL;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;



/**
 * Загружаем файлы из заданного jar-архива
 * Классы должны относится к заданному пакету - пример валидации при загрузке
 *
 */

public class MyJarClassLoader extends ClassLoader {
    private HashMap<String, Class<?>> cache = new HashMap<String, Class<?>>();
    private String jarFileName;
    private String packageName;
    private static String WARNING = "Warning : No jar file found. Packet unmarshalling"+
     " won't be possible. Please verify your classpath";

    public MyJarClassLoader(String jarFileName, String packageName) {
      this.jarFileName = jarFileName;
      this.packageName = packageName;
      cacheClasses();
    }

    /**
     * При создании загрузчика извлекаем все классы из jar и кэшируем в памяти
     *
     */

    private void cacheClasses() {
        try {
          System.out.println("Open jar "+jarFileName);
//          jar:http://www.foo.com/bar/baz.jar!
          URL url = new URL("jar:"+jarFileName+"!/");
          JarURLConnection jarConnection = (JarURLConnection)url.openConnection();
          JarFile jarFile = jarConnection.getJarFile();
//          JarFile jarFile = new JarFile(jarFileName);
//          Manifest manifest = jarConnection.getManifest();
          Enumeration entries = jarFile.entries();
          while (entries.hasMoreElements()) {
            JarEntry jarEntry = (JarEntry) entries.nextElement();
            System.out.println("jarEntry.getName()="+jarEntry.getName());
            // Одно из назначений хорошего загрузчика - валидация классов на этапе загрузки
            if (match(normalize(jarEntry.getName()), packageName)) {
              System.out.println("Before load in memory");
              byte[] classData = loadClassData(jarFile, jarEntry);
              System.out.println("after load in memory");
              if (classData != null) {
                System.out.println("Before define ");
                System.out.println("stripClassName(normalize(jarEntry.getName()))="+
                stripClassName(normalize(jarEntry.getName())));
                System.out.println("normalize(jarEntry.getName())="+normalize(jarEntry.getName()));
                System.out.println("jarEntry.getName()="+jarEntry.getName());
                Class<?> clazz = defineClass(stripClassName(normalize(jarEntry.getName())), classData, 0, classData.length);
                System.out.println("After define ");
                cache.put(clazz.getName(), clazz);
                System.out.println("After cache ");
                System.out.println("== class " + clazz.getName() + " loaded in cache");
              }
            }
          }
          jarFile.close();
        }
        catch (IOException IOE) {
          // Просто выведем сообщение об ошибке
          System.out.println(WARNING);
        }
      }

    /**
     * Собственно метод, который и реализует загрузку класса
     *
     */
    public synchronized Class<?> loadClass(String name) throws ClassNotFoundException {
      Class<?> result = cache.get(name);
      // Возможно класс вызывается не по полному имени - добавим имя пакета
      if (result == null) result = cache.get(packageName + "." + name);
      // Если класса нет в кэше то возможно он системный
      if (result == null) result = super.findSystemClass(name);
      System.out.println("== loadClass(" + name + ")");
      return result;
    }

    /**
     * Получаем каноническое имя класса
     * @param className
     * @return
     */

    private String stripClassName(String className) {  return className.substring(0, className.length() - 6);  }

    /**
     * Преобразуем имя в файловой системе в имя класса
     * (заменяем слэши на точки)
     *
     * @param className
     * @return
     */

    private String normalize(String className) { return className.replace('/', '.'); }

    /**
     * Валидация класса - проверят принадлежит ли класс заданному пакету и имеет ли
     * он расширение .class
     *..
     * @param className
     * @param packageName
     * @return
     */
    private boolean match(String className, String packageName)
     { return className.startsWith(packageName) && className.endsWith(".class");  }
    /**
     * Извлекаем файл из заданного JarEntry
     *
     * @param jarFile - файл jar-архива из которого извлекаем нужный файл
     * @param jarEntry - jar-сущность которую извлекаем
     * @return null если невозможно прочесть файл
     */

    private byte[] loadClassData(JarFile jarFile, JarEntry jarEntry) throws IOException {
      long size = jarEntry.getSize();
      System.out.println(jarEntry.getName()+" size="+size);
      if (size == -1 || size == 0) return null;
      byte[] data = new byte[(int)size];
      InputStream in = jarFile.getInputStream(jarEntry);
      in.read(data);
      System.out.println("data.length="+data.length);
      return data;
    }
}

    